bits 16
org	0x0


	jmp	BOOT_SEG:start

BOOT_SEG	equ 0x7c0	; segment address of bootloader 0x7c00
SETUP_SEG	equ 0x900	; segment address of Setup.asm 0x9000
KERNEL_SEG	equ 0x100	; segment address of KERNEL image 0x100000=1M

bootDrv		db 0		; variable to save floppy drive number

sectTrack	dw 18		; number of sectors of each track
headCount	dw 2		; number of headers of floppy drive

currentSect	dw 1		; current read sector 
readSect	db 0		; sector number to read

kernelSect	db KERNEL_SIZE		; size of KERNEL image


start:
	; set segment address, 0x7c00 same with code segment
	mov	ax, cs
	mov	ds, ax
	mov	ss, ax
	; set stack address
	; The size of stack is 0x1400(5120 bytes=5K)
	; stack start at 0x7c00+0x1400 = 0x9000
	mov	sp, 0x1400

	; save boot driver number...
	mov	[bootDrv], dl

	; load setup file to 0x900:0...
	mov	ax, SETUP_SEG
	mov	es, ax
	mov	ax, [currentSect]
	call	read_sect

	; display system beginning message...
	mov	si, startMsg
	call	put_char

	; load kernel image...
	call	load_kernel

	; jump to Setup.asm
	jmp	0x0:0x9000


; load kernel image
load_kernel:
	mov	ax, KERNEL_SEG
	mov	es, ax

_load_kernel_1:
	mov	ax, [currentSect]	; read one sector
	call	read_sect
	
	dec	byte[kernelSect]	; count of KERNEL size
	jz	_load_kernel
	
	; one sector = 512 bytes => 0x200
	mov	ax, es
	add	ax, 0x20	; read next 512 bytes
	mov	es, ax
	
	jmp	_load_kernel_1

_load_kernel:
	ret


; read a sector
; 1. compute current head, track, sector numbers with currentSect
; 2. compute 
; arguments : es segment address to store sector
;             ax sector number to read
read_sect:
	; compute absolute sector number and relative track number
	; relative sector number(ax) / sectors per track(si)
	; = relarive track number(ax), absolute sector number of track(dx)
	; eg) 1/18 = 0 track, 1 sector
	mov	si, [sectTrack]
	xor	dx, dx
	div	si				; ax=track, dx=sector
	

	inc	dl
	mov	[readSect], dl	; save sector number to read

	; compute head number and absolute track number
	; relative track number(ax) / head
	; = absolute track number(ax), head number(dx)
	xor	dx, dx
	div	word[headCount]	; divide with head count


	;-----------------------------------------------
	; read sector to use interrupt 0x13 - service 0x2
	; al = sector count
	; es = segment address
	; bx = offset address
	; cl = sector number
	; ch = track number
	; dl = drive
	; dh = head number
	;---------------------------------------------
	mov	dh, [bootDrv]
	xchg dl, dh
	mov	cx, ax			; store track number to cx
	xchg cl, ch			; track number is stored in ch
	shl	cl, 6			; ??
	or	cl, [readSect]		; ??
	mov	al, 1			; sector count to read
	mov	ah, 2			; service number 0x2
	xor	bx, bx			; offset, segment = es
	int	0x13
	jc	boot_fail		; if carry, error occured
	inc	byte[currentSect]	; incread current sector
	ret


; display message...
put_char:
	lodsb
	or	al, al
	jz	_put_char

	mov	ah, 0xe
	mov	bx, 0x7
	int	0x10
	jmp	put_char

_put_char:
	ret


; boot failed...
boot_fail:
	mov	si, failMsg
	call	put_char
_fail:
	jmp	_fail


startMsg	db 'Start csOS...', 0xa, 0xd, 0x0
failMsg		db 'Boot failed...', 0x0


	; mbr flag set...
	times 510-($-$$) db 0
	dw 0xaa55
